package org.hisp.dhis.mobile.connection;

/*
 * Copyright (c) 2004-2011, University of Oslo All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met: * Redistributions of source code must retain the above copyright notice, this list of
 * conditions and the following disclaimer. * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation and/or other materials provided with the
 * distribution. * Neither the name of the HISP project nor the names of its contributors may be used to endorse or
 * promote products derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

import java.io.IOException;
import javax.microedition.io.Connector;
import javax.microedition.io.HttpConnection;
import javax.microedition.io.HttpsConnection;
import org.hisp.dhis.mobile.connection.task.AbstractTask;
import org.hisp.dhis.mobile.connection.task.DownloadAllResourceTask;
import org.hisp.dhis.mobile.connection.task.LoginTask;
import org.hisp.dhis.mobile.connection.task.UpdateActivityPlanTask;
import org.hisp.dhis.mobile.connection.task.UpdateDataSetTask;
import org.hisp.dhis.mobile.connection.task.UploadActivityValueTask;
import org.hisp.dhis.mobile.connection.task.UploadDataSetValueTask;
import org.hisp.dhis.mobile.connection.task.UploadPatientIdentifierNumberTask;
import org.hisp.dhis.mobile.midlet.DHISMIDlet;
import org.hisp.dhis.mobile.model.Activity;
import org.hisp.dhis.mobile.model.ActivityValue;
import org.hisp.dhis.mobile.model.DataSetValue;
import org.hisp.dhis.mobile.model.OrgUnit;

public class ConnectionManager

{
    public static final String MEDIATYPE_MOBILE_SERIALIZED = "application/vnd.org.dhis2.mobile+serialized";

    // Class variables

    private static DHISMIDlet dhisMIDlet;

    private static String url;

    private static String userName;

    private static String password;

    private static String ua;

    private static String locale;

    private static OrgUnit orgUnit;

    public ConnectionManager()
    {

    }

    public static void init( org.hisp.dhis.mobile.midlet.DHISMIDlet dhisMIDlet, String url, String userName,
        String password, String locale, OrgUnit orgUnit )
    {
        ConnectionManager.dhisMIDlet = dhisMIDlet;
        ConnectionManager.url = url;
        ConnectionManager.userName = userName;
        ConnectionManager.password = password;
        ConnectionManager.locale = locale;
        ConnectionManager.orgUnit = orgUnit;

        ua = "Profile/" + System.getProperty( "microedition.profiles" ) + " Configuration/"
            + System.getProperty( "microedition.configuration" );

        DHISMIDlet.debug( "Initialize Connection Manager: \n" + "URL: " + url + "\n" + "User: " + userName + "\n"
            + "Pass: " + password + "\n" + "Locale: " + locale );
    }

    public static HttpConnection createConnection()
    {
        HttpConnection connection = null;
        try
        {
            connection = getConnection( url );
            connection.setRequestProperty( "User-Agent", ua );
            connection.setRequestProperty( "Accept-Language", locale );
            connection.setRequestProperty( "Content-Type", MEDIATYPE_MOBILE_SERIALIZED );
            connection.setRequestProperty( "Accept", MEDIATYPE_MOBILE_SERIALIZED );

            // set HTTP basic authentication

            if ( ConnectionManager.userName != null && password != null )
            {
                byte[] auth = (userName + ":" + password).getBytes();
                connection.setRequestProperty( "Authorization", "Basic " + Base64.encode( auth, 0, auth.length ) );
            }
        }
        catch ( IOException e )
        {
            e.printStackTrace();
        }
        return connection;
    }

    public static HttpConnection getConnection( String url )
        throws IOException
    {
        if ( url.startsWith( "https://" ) )
        {
            return (HttpsConnection) Connector.open( url, Connector.READ_WRITE, true );
        }
        else
        {
            return (HttpConnection) Connector.open( url, Connector.READ_WRITE, true );
        }
    }

    public static void login()
    {
        runTask( new LoginTask() );
    }

    public static void downloadAllResource()
    {
        runTask( new DownloadAllResourceTask() );
    }

    public static void updateDataSet()
    {
        runTask( new UpdateDataSetTask() );
    }

    public static void uploadDataSetValue( DataSetValue dataSetValue )
    {
        UploadDataSetValueTask uploadDataSetValueTask = new UploadDataSetValueTask();
        uploadDataSetValueTask.setDataSetValue( dataSetValue );
        runTask( uploadDataSetValueTask );
    }

    public static void uploadActivityValue( ActivityValue activityValue, Activity activity )
    {
        UploadActivityValueTask uploadActivityValueTask = new UploadActivityValueTask();
        uploadActivityValueTask.setActivity( activity );
        uploadActivityValueTask.setActivityValue( activityValue );
        runTask( uploadActivityValueTask );
    }

    public static void updateActivityPlan()
    {
        runTask( new UpdateActivityPlanTask() );
    }

    public static void uploadBeneficiaryIdentifier( String identifier )
    {
        UploadPatientIdentifierNumberTask uploadPatientIdentifierNumberTask = new UploadPatientIdentifierNumberTask();
        uploadPatientIdentifierNumberTask.setPatientIdentifier( identifier );
        runTask( uploadPatientIdentifierNumberTask );
    }

    public static void runTask( AbstractTask task )
    {
        new Thread( task ).start();
    }

    public static DHISMIDlet getDhisMIDlet()
    {
        return dhisMIDlet;
    }

    public static void setDhisMIDlet( DHISMIDlet dhisMIDlet )
    {
        ConnectionManager.dhisMIDlet = dhisMIDlet;
    }

    public static String getUrl()
    {
        return url;
    }

    public static void setUrl( String url )
    {
        ConnectionManager.url = url;
    }

    public static String getUserName()
    {
        return userName;
    }

    public static void setUserName( String userName )
    {
        ConnectionManager.userName = userName;
    }

    public static String getPassword()
    {
        return password;
    }

    public static void setPassword( String password )
    {
        ConnectionManager.password = password;
    }

    public static String getUa()
    {
        return ua;
    }

    public static void setUa( String ua )
    {
        ConnectionManager.ua = ua;
    }

    public static String getLocale()
    {
        return locale;
    }

    public static void setLocale( String locale )
    {
        ConnectionManager.locale = locale;
    }

    public static OrgUnit getOrgUnit()
    {
        return orgUnit;
    }

    public static void setOrgUnit( OrgUnit orgUnit )
    {
        ConnectionManager.orgUnit = orgUnit;
    }

    public static String getMediatypeMobileSerialized()
    {
        return MEDIATYPE_MOBILE_SERIALIZED;
    }
}
