package org.hisp.dhis.mobile.midlet;

/*
 * Copyright (c) 2004-2011, University of Oslo All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met: * Redistributions of source code must retain the above copyright notice, this list of
 * conditions and the following disclaimer. * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation and/or other materials provided with the
 * distribution. * Neither the name of the HISP project nor the names of its contributors may be used to endorse or
 * promote products derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

import java.io.IOException;
import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;
import javax.microedition.rms.RecordStoreException;
import org.hisp.dhis.mobile.model.TextMapping;
import org.hisp.dhis.mobile.recordstore.SettingRecordStore;
import org.hisp.dhis.mobile.ui.Text;
import org.hisp.dhis.mobile.ui.TextMappingLoader;
import org.hisp.dhis.mobile.view.LoginView;
import org.hisp.dhis.mobile.view.OrgUnitSelectView;
import org.hisp.dhis.mobile.view.PinView;
import org.hisp.dhis.mobile.view.SettingView;
import org.hisp.dhis.mobile.view.SplashScreenView;
import org.hisp.dhis.mobile.view.WaitingView;

public class DHISMIDlet
    extends MIDlet
{
    // Debug Mode

    public static final boolean DEBUG = true;

    public static final String DEBUG_PREFIX = "[DEBUG] ";

    // Flag

    public static boolean isDownloading;

    // Common MIDlet Variables

    private SplashScreenView splashScreenView;

    private WaitingView waitingView;

    private LoginView loginView;

    private OrgUnitSelectView orgUnitSelectView;

    private PinView pinView;

    private SettingView settingView;

    // Others

    public static String DEFAULT_LOCALE = "en-GB";

    public static final String BLANK = "";

    // For text value

    public static final int TEXT_NUMERIC = 8;

    public static final int TEXT_STRING = 64;

    protected void destroyApp( boolean b )
        throws MIDletStateChangeException
    {
        // TODO Auto-generated method stub
    }

    protected void pauseApp()
    {
        // TODO Auto-generated method stub
    }

    protected void startApp()
        throws MIDletStateChangeException
    {
        try
        {
            this.loadLanguageLocale();
        }
        catch ( RecordStoreException e )
        {
            e.printStackTrace();
        }
        catch ( IOException e )
        {
            e.printStackTrace();
        }
        this.getSplashScreenView().showView();
    }

    private void loadLanguageLocale()
        throws RecordStoreException, IOException
    {
        SettingRecordStore settingRecordStore = new SettingRecordStore();
        String locale = settingRecordStore.get( SettingRecordStore.LOCALE );

        if ( locale.equals( "" ) )
        {
            TextMapping textMapping = TextMappingLoader.load( DEFAULT_LOCALE );
            Text.setTextMapping( textMapping );
        }
        else
        {
            TextMapping textMapping = TextMappingLoader.load( locale );
            Text.setTextMapping( textMapping );
        }

    }

    public void exitMIDlet()
    {
        try
        {
            this.destroyApp( true );
        }
        catch ( MIDletStateChangeException e )
        {
            e.printStackTrace();
        }
        notifyDestroyed();
    }

    public static void debug( String message )
    {
        if ( DEBUG )
        {
            System.out.println( DEBUG_PREFIX + message );
        }
    }

    public SplashScreenView getSplashScreenView()
    {
        if ( splashScreenView == null )
        {
            splashScreenView = new SplashScreenView( this );
        }
        return splashScreenView;
    }

    public void setSplashScreenView( SplashScreenView splashScreenView )
    {
        this.splashScreenView = splashScreenView;
    }

    public WaitingView getWaitingView()
    {
        if ( waitingView == null )
        {
            waitingView = new WaitingView( this );
        }
        return waitingView;
    }

    public void setWaitingView( WaitingView waitingView )
    {
        this.waitingView = waitingView;
    }

    public LoginView getLoginView()
    {
        if ( loginView == null )
        {
            loginView = new LoginView( this );
        }
        return loginView;
    }

    public OrgUnitSelectView getOrgUnitSelectView()
    {
        if ( orgUnitSelectView == null )
        {
            orgUnitSelectView = new OrgUnitSelectView( this );
        }
        return orgUnitSelectView;
    }

    public void setOrgUnitSelectView( OrgUnitSelectView orgUnitSelectView )
    {
        this.orgUnitSelectView = orgUnitSelectView;
    }

    public void setLoginView( LoginView loginView )
    {
        this.loginView = loginView;
    }

    public PinView getPinView()
    {
        if ( pinView == null )
        {
            pinView = new PinView( this );
        }
        return pinView;
    }

    public void setPinView( PinView pinView )
    {
        this.pinView = pinView;
    }

    public SettingView getSettingView()
    {
        if ( settingView == null )
        {
            settingView = new SettingView( this );
        }
        return settingView;
    }

    public void setSettingView( SettingView settingView )
    {
        this.settingView = settingView;
    }

    public static boolean isDownloading()
    {
        return isDownloading;
    }

    public static void setDownloading( boolean isDownloading )
    {
        DHISMIDlet.isDownloading = isDownloading;
    }
}
